SystemOrganization addCategory: #'SystemEditor-Tests'!
SystemOrganization addCategory: #'SystemEditor-Editors'!

Behavior subclass: #MetaclassEditor
	instanceVariableNames: 'editor instVarNames'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Editors'!

!MetaclassEditor class methodsFor: 'instance creation' stamp: 'cwp 10/23/2005 14:38'!
new
	| instance |
	instance _ self basicNew.
	instance
		superclass: ClassEditor
		methodDictionary: MethodDictionary new
		format: ClassEditor format.
	^ instance! !

!MetaclassEditor methodsFor: 'editing' stamp: 'cwp 10/23/2005 23:16'!
addInstVarName: aString 
	instVarNames ifNil: [instVarNames _ self subject instVarNames].
	instVarNames _ instVarNames copyWith: aString! !

!MetaclassEditor methodsFor: 'reflecting' stamp: 'cwp 10/24/2005 18:21'!
classPool
	^ editor classPool! !

!MetaclassEditor methodsFor: 'building' stamp: 'cwp 10/24/2005 07:33'!
edBuild
	| result |
	result _ self subject clone.
	result setInstVarNames: self instVarNames.
	^ result! !

!MetaclassEditor methodsFor: 'reflecting' stamp: 'cwp 10/24/2005 07:40'!
format
	"Override the implemenation in Behavior. That will return the format of ClassEditor,
	but we want the format of the subject's meta class"

	| superFormat |
	superFormat := ClassFormat fromBits: self subject superclass format.
	^ (superFormat namedSubclass: self instVarNames size) bits! !

!MetaclassEditor methodsFor: 'reflecting' stamp: 'cwp 10/23/2005 23:17'!
instVarNames
	"Override the implemenation in Behavior. That will return the inst vars of 
	ClassEditor, but we want the inst vars of the subject's meta class."

	^ instVarNames ifNil: [self subject instVarNames]! !

!MetaclassEditor methodsFor: 'metaclass' stamp: 'cwp 10/23/2005 14:43'!
new
	^ editor _ self basicNew! !

!MetaclassEditor methodsFor: 'printing' stamp: 'cwp 10/23/2005 14:45'!
printOn: aStream
	aStream
		nextPutAll: 'a MetaclassEditor on: ';
		print: editor subject ! !

!MetaclassEditor methodsFor: 'accessing' stamp: 'cwp 10/23/2005 14:45'!
subject
	^ editor subject class! !

!MetaclassEditor methodsFor: 'reflecting' stamp: 'cwp 10/24/2005 07:46'!
superclass
	^ self subject superclass! !

TestCase subclass: #ClassEditorTest
	instanceVariableNames: 'editor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Tests'!

ClassEditorTest subclass: #ClassEditorBuildTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Tests'!

!ClassEditorBuildTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 22:07'!
test01NoChange
	"Since there was no change, we don't have to rebuild the class at all."

	self assert: editor edBuild == editor subject.! !

!ClassEditorBuildTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 22:08'!
test02AddInstVar
	
	| result |
	editor addInstVarName: 'gamma'.
	result _ editor edBuild.
	self deny: result = editor subject.
	self assert: result name == editor subject name.
	self assert: result instVarNames = editor instVarNames.
	self assert: result format = editor format.! !

!ClassEditorBuildTest methodsFor: 'tests' stamp: 'cwp 10/24/2005 08:07'!
test03Superclass
	
	| result |
	editor superclass: self class.
	result _ editor edBuild.
	self deny: result = editor subject.
	self assert: result superclass == self class.! !

!ClassEditorBuildTest methodsFor: 'tests' stamp: 'cwp 10/24/2005 08:14'!
test04AddClassVar
	
	| result |
	editor addClassVarName: 'Gimel'.
	result _ editor edBuild.
	self deny: result = editor subject.
	self assert: result classVarNames = editor classVarNames! !

!ClassEditorBuildTest methodsFor: 'tests' stamp: 'cwp 10/24/2005 18:37'!
test05RemoveClassVar
	
	| result |
	editor removeClassVarName: 'Beth'.
	result _ editor edBuild.
	self deny: result = editor subject.
	self assert: result classVarNames = editor classVarNames! !

!ClassEditorBuildTest methodsFor: 'tests' stamp: 'cwp 10/24/2005 18:40'!
test06PreserveClassVar
	
	| result object |
	object _ Object new.
	editor removeClassVarName: 'Beth'.
	editor subject aleph: object.
	result _ editor edBuild.
	self deny: result = editor subject.
	self assert: result aleph == object! !

!ClassEditorBuildTest methodsFor: 'tests' stamp: 'cwp 10/24/2005 18:56'!
test07AddSharedPool
	
	| result pool |
	pool _ Dictionary new.
	editor addSharedPool: pool.
	result _ editor edBuild.
	self deny: result = editor subject.
	self assert: (result sharedPools includes: pool).
	self deny: (editor subject sharedPools includes: pool).! !

!ClassEditorBuildTest methodsFor: 'tests' stamp: 'cwp 10/24/2005 19:00'!
test08RemoveSharedPool
	
	| result |
	editor removeSharedPool: EditorSubjectPool.
	result _ editor edBuild.
	self deny: result = editor subject.
	self deny: (result sharedPools includes: EditorSubjectPool).
	self assert: (editor subject sharedPools includes: EditorSubjectPool).! !

ClassEditorTest subclass: #ClassEditorOntologyTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Tests'!

!ClassEditorOntologyTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 15:06'!
test01Creation
	"This invariant allows editors and their subject classes to use
	exactly the same reflection protocol, so that the only modification
	required to allow tools to modify the system atomically is to send
	#commit when all modifications are complete."

	self assert: editor subject == EditorSubject.
	self assert: editor class subject == EditorSubject class! !

!ClassEditorOntologyTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 15:06'!
test02Printing
	"This makes debugging easier."
	
	self assert: editor printString = 'a ClassEditor on: EditorSubject'.
	self assert: editor class printString = 'a MetaclassEditor on: EditorSubject'.! !

!ClassEditorOntologyTest methodsFor: 'tests' stamp: 'cwp 10/24/2005 07:47'!
test03ClassReferences

	"The instance is a strange beast. Its class is created on the fly, and is
	both an instance of MetaclassEditor and a subclass of ClassEditor. 
	See #testCreation." 
	self assert: editor class class == MetaclassEditor.
	self assert: (editor class instVarNamed: 'superclass') == ClassEditor.


	"editor is an instance of a light-weight class, so we don't register it
	with its superclass. In fact, the only thing that keeps it from being
	garbage collected is the class pointer from editor."
	self deny: (ClassEditor subclasses includes: editor class).

! !

!ClassEditorOntologyTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 15:07'!
test04IsKindOfClass
	"A class and its editor are polymorphic, but this kind of
	explicit test can tell the difference."

	self deny: (editor class isKindOf: Class)! !

!ClassEditorOntologyTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 15:24'!
test05ClassPool
	"To have DoIts be executed in the right context by a debugger or inspector,
	the generated class needs to be adoptable by FakeClassPool."

	self 
		shouldnt: [FakeClassPool adopt: editor class]
		raise: Error.! !

!ClassEditorOntologyTest methodsFor: 'tests' stamp: 'cwp 10/24/2005 07:36'!
test06Format
	"Despite having an instance variable call 'format' 
	MetaclassEditors should answer the format of the 
	subject's metaclass, not the format of ClassEditor class."

	self assert: editor class format = EditorSubject class format! !

!ClassEditorOntologyTest methodsFor: 'tests' stamp: 'cwp 10/24/2005 07:43'!
test07Superclass
	"Despite having an instance variable call 'superclass' 
	MetaclassEditors should answer the superclass of the 
	subject's class, not ClassEditor."

	self assert: editor class superclass = EditorSubject class superclass! !

ClassEditorTest subclass: #ClassEditorProtocolTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Tests'!

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 15:40'!
test01InstVarNames
	self assert: editor instVarNames = editor subject instVarNames! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 15:36'!
test02AddInstVarName
	editor addInstVarName: 'gamma'.
	self assert: editor instVarNames = (editor subject instVarNames copyWith: 'gamma')! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 15:58'!
test03RemoveClassVarName
	editor removeClassVarName: 'Beth'.
	self assert: editor classVarNames = (editor subject classVarNames copyWithout: 'Beth')! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 15:40'!
test04ClassVarNames
	self assert: editor classVarNames = editor subject classVarNames! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/24/2005 08:23'!
test05AddClassVarName
	editor addClassVarName: #Gimel.
	self assert: editor classVarNames = (editor subject classVarNames copyWith: #Gimel)! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 15:59'!
test06RemoveClassVarName
	editor removeClassVarName: 'Beth'.
	self assert: editor classVarNames = (editor subject classVarNames copyWithout: 'Beth')! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 16:00'!
test07SharedPools
	self assert: editor sharedPools = editor subject sharedPools! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 16:47'!
test08AddSharedPool
	| pool |
	pool _ Dictionary new.
	editor addSharedPool: pool.
	self assert: editor sharedPools = (editor subject sharedPools copyWith: pool)! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/24/2005 07:51'!
test09RemoveSharedPool
	editor removeSharedPool: EditorSubjectPool.
	self assert: editor sharedPools = (editor subject sharedPools copyWithout: EditorSubjectPool)! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/24/2005 07:53'!
test10Superclass
	editor superclass: self class.
	self assert: editor superclass == self class.! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/24/2005 08:27'!
test11ClassVarNamesInterned
	editor addClassVarName: 'Gimel'.
	self assert: (editor classVarNames anySatisfy: [:ea | ea == #Gimel])! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/25/2005 23:00'!
test12AddSelector
	self deny: (editor includesSelector: #one).
	editor 
		compile: 'one ^ 1'
		classified: 'numbers'
		withStamp: 'cwp 5/30/2004 09:27'
		notifying: nil
		logSource: true.
	self assert: (editor includesSelector: #one)! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/25/2005 23:01'!
test13RemoveSelector
	editor removeSelector: #one.
	self deny: (editor includesSelector: #one)! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/25/2005 23:07'!
test14AddAndRemoveSelector
	editor 
		compile: 'one ^ 1'
		classified: 'numbers'
		withStamp: 'cwp 5/30/2004 09:27'
		notifying: nil
		logSource: true.
	editor removeSelector: #one.
	self deny: (editor includesSelector: #one).
	! !

!ClassEditorTest class methodsFor: 'testing' stamp: 'cwp 10/23/2005 15:31'!
isAbtract
	^ self name == #ClassEditorTest! !

!ClassEditorTest methodsFor: 'running' stamp: 'cwp 10/23/2005 15:29'!
setUp
	editor := ClassEditor on: EditorSubject! !

TestCase subclass: #ClassFormatTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Tests'!

!ClassFormatTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 18:27'!
testBitsWithInstSize
	| format |
	format _ ClassFormat bytes.
	self should: [format withInstSize: 2] raise: Error! !

!ClassFormatTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 18:28'!
testBytes
	| format |
	format := ClassFormat bytes.
	self assert: format instSize = 0.
	self assert: format isVariable.
	self assert: format isBits.
	self assert: format isBytes.
	self deny: format isFixed.
	self deny: format isPointers.
	self deny: format isWords.
	self deny: format isWeak
! !

!ClassFormatTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 18:27'!
testIndexed
	| format |
	format := ClassFormat indexed.
	self assert: format instSize = 0.
	self assert: format isPointers.
	self assert: format isWords.
	self assert: format isVariable.
	self deny: format isFixed.
	self deny: format isBits.
	self deny: format isBytes.
	self deny: format isWeak
! !

!ClassFormatTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 18:28'!
testIndexedSubclass
	| superFormat subFormat |
	superFormat := ClassFormat named: 3.
	subFormat := superFormat indexedSubclass: 2.
	self assert: superFormat instSize = 3.
	self assert: subFormat instSize = 5.
	self assert: subFormat isVariable.
	self assert: subFormat isPointers.
	self assert: subFormat isWords.
	self deny: subFormat isFixed.
	self deny: subFormat isBits.
	self deny: subFormat isBytes.
	self deny: subFormat isWeak! !

!ClassFormatTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 18:28'!
testIndexedWithInstSize
	| old new |
	old _ ClassFormat indexed: 3.
	new _ old withInstSize: 5.
	self assert: new instSize = 5.
	self assert: new instSpec = old instSpec.
	self assert: new cClassIndex = old cClassIndex! !

!ClassFormatTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 18:27'!
testNamed
	| format |
	format := ClassFormat named: 3.
	self assert: format instSize = 3.
	self assert: format isFixed.
	self assert: format isPointers.
	self assert: format isWords.
	self deny: format isVariable.
	self deny: format isBits.
	self deny: format isBytes.
	self deny: format isWeak! !

!ClassFormatTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 18:28'!
testNamedAndIndexed
	| format |
	format := ClassFormat indexed: 3.
	self assert: format instSize = 3.
	self assert: format isPointers.
	self assert: format isWords.
	self assert: format isVariable.
	self deny: format isFixed.
	self deny: format isBits.
	self deny: format isBytes.
	self deny: format isWeak
! !

!ClassFormatTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 18:28'!
testNamedSubclass
	| superFormat subFormat |
	superFormat := ClassFormat named: 3.
	subFormat := superFormat namedSubclass: 2.
	self assert: superFormat instSize = 3.
	self assert: subFormat instSize = 5.
	self assert: subFormat isFixed.
	self assert: subFormat isPointers.
	self assert: subFormat isWords.
	self deny: subFormat isVariable.
	self deny: subFormat isBits.
	self deny: subFormat isBytes.
	self deny: subFormat isWeak! !

!ClassFormatTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 18:28'!
testNamedWithInstSize
	| old new |
	old _ ClassFormat named: 3.
	new _ old withInstSize: 5.
	self assert: new instSize = 5.
	self assert: new instSpec = old instSpec.
	self assert: new cClassIndex = old cClassIndex! !

!ClassFormatTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 18:28'!
testNoState
	| format |
	format := ClassFormat noState.
	self assert: format instSize = 0.
	self assert: format isFixed.
	self assert: format isPointers.
	self assert: format isWords.
	self deny: format isVariable.
	self deny: format isBits.
	self deny: format isBytes.
	self deny: format isWeak! !

!ClassFormatTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 18:28'!
testWeak
	| format |
	format := ClassFormat weak.
	self assert: format instSize = 0.
	self assert: format isVariable.
	self assert: format isPointers.
	self assert: format isWords.
	self assert: format isWeak.
	self deny: format isFixed.
	self deny: format isBits.
	self deny: format isBytes.
! !

!ClassFormatTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 18:28'!
testWords
	| format |
	format := ClassFormat words.
	self assert: format instSize = 0.
	self assert: format isVariable.
	self assert: format isBits.
	self assert: format isWords.
	self deny: format isFixed.
	self deny: format isPointers.
	self deny: format isBytes.
	self deny: format isWeak
! !

TestCase subclass: #MetaclassEditorTest
	instanceVariableNames: 'editor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Tests'!

MetaclassEditorTest subclass: #MetaclassEditorBuildTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Tests'!

!MetaclassEditorBuildTest methodsFor: 'as yet unclassified' stamp: 'cwp 10/24/2005 08:02'!
test01InstVars
	| result |
	result _ editor edBuild.
	self assert: result instVarNames = editor instVarNames! !

MetaclassEditorTest subclass: #MetaclassEditorProtocolTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Tests'!

!MetaclassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 22:50'!
test01InstVarNames
	self assert: editor instVarNames = editor subject instVarNames! !

!MetaclassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 23:14'!
test02AddInstVarName
	editor addInstVarName: 'psi'.
	self assert: editor instVarNames = (editor subject instVarNames copyWith: 'psi')! !

!MetaclassEditorTest methodsFor: 'running' stamp: 'cwp 10/24/2005 07:57'!
setUp
	editor _ (ClassEditor on: EditorSubject) class! !

SharedPool subclass: #EditorSubjectPool
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Tests'!

Object subclass: #ClassEditor
	instanceVariableNames: 'subject superclass instVarNames classVarNames sharedPools methods'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Editors'!

!ClassEditor class methodsFor: 'instance creation' stamp: 'cwp 10/23/2005 14:35'!
on: aClass
	| meta instance |
	meta _ MetaclassEditor new.
	instance _ meta new.
	instance setSubject: aClass.
	^ instance! !

!ClassEditor methodsFor: 'editing' stamp: 'cwp 10/24/2005 08:27'!
addClassVarName: aSymbol
	classVarNames ifNil: [classVarNames _ subject classVarNames].
	classVarNames _ classVarNames copyWith: aSymbol asSymbol! !

!ClassEditor methodsFor: 'editing' stamp: 'cwp 10/23/2005 15:36'!
addInstVarName: aString 
	instVarNames ifNil: [instVarNames _ subject instVarNames].
	instVarNames _ instVarNames copyWith: aString! !

!ClassEditor methodsFor: 'editing' stamp: 'cwp 10/23/2005 16:03'!
addSharedPool: aSharedPool 
	sharedPools ifNil: [sharedPools _ subject sharedPools].
	sharedPools _ sharedPools copyWith: aSharedPool! !

!ClassEditor methodsFor: 'reflecting' stamp: 'cwp 10/24/2005 18:28'!
classPool
	| pool |
	pool _ self subject classPool copy.
	classVarNames ifNil: [^ pool].
	pool keys do: [:ea | (classVarNames includes: ea) ifFalse: [pool removeKey: ea]].
	classVarNames do: [:ea | (pool includesKey: ea) ifFalse: [pool at: ea put: nil]].
	^ pool! !

!ClassEditor methodsFor: 'reflecting' stamp: 'cwp 10/23/2005 15:41'!
classVarNames
	^ classVarNames ifNil: [subject classVarNames]! !

!ClassEditor methodsFor: 'editing' stamp: 'cwp 10/25/2005 22:50'!
compile: source classified: category withStamp: aString notifying: anObject logSource: aBoolean
	| cm |
	cm := ModifiedMethod
			source: source
			classified: category
			stamp: aString
			notifying: anObject
			logging: aBoolean.
	methods ifNil: [methods _ MethodDictionaryEditor on: self subject methodDictionary].
	methods add: cm.! !

!ClassEditor methodsFor: 'building' stamp: 'cwp 10/24/2005 19:02'!
edBuild
	| meta result |
	self edRequiresBuild ifFalse: [^ subject].
	meta _ self class edBuild.
	result _ meta adoptInstance: subject from: subject class.
	result
		superclass: self superclass
		methodDictionary: MethodDictionary new
		format: self format;
		setInstVarNames: self instVarNames;
		classPoolFrom: self;
		instVarNamed: #sharedPools put: self sharedPools.

	^ result! !

!ClassEditor methodsFor: 'building' stamp: 'cwp 10/24/2005 18:52'!
edRequiresBuild
	^ superclass ~~ nil
		| instVarNames ~~ nil
		| classVarNames ~~ nil
		| sharedPools ~~ nil! !

!ClassEditor methodsFor: 'reflecting' stamp: 'cwp 10/24/2005 08:12'!
format
	| superFormat |
	superFormat _ ClassFormat fromBits: self superclass format.
	^ (superFormat namedSubclass: self instVarNames size) bits! !

!ClassEditor methodsFor: 'reflecting' stamp: 'cwp 10/25/2005 22:35'!
includesSelector: aSelector
	^ (methods ifNil: [self subject] ifNotNil: [methods])
		includesSelector: aSelector! !

!ClassEditor methodsFor: 'reflecting' stamp: 'cwp 10/23/2005 15:37'!
instVarNames
	^ instVarNames ifNil: [subject instVarNames]! !

!ClassEditor methodsFor: 'printing' stamp: 'cwp 10/23/2005 02:58'!
printOn: aStream
	aStream 
		nextPutAll: 'a ClassEditor on: ';
		nextPutAll: subject name! !

!ClassEditor methodsFor: 'editing' stamp: 'cwp 10/23/2005 15:58'!
removeClassVarName: aString 
	classVarNames ifNil: [classVarNames _ subject classVarNames].
	classVarNames _ classVarNames copyWithout: aString! !

!ClassEditor methodsFor: 'editing' stamp: 'cwp 10/23/2005 15:39'!
removeInstVarName: aString 
	instVarNames ifNil: [instVarNames _ subject instVarNames].
	instVarNames _ instVarNames copyWithout: aString! !

!ClassEditor methodsFor: 'reflecting' stamp: 'cwp 10/25/2005 23:03'!
removeSelector: aSymbol 
	methods ifNil: [methods _ MethodDictionaryEditor on: self subject methodDictionary].
	methods remove: aSymbol.! !

!ClassEditor methodsFor: 'editing' stamp: 'cwp 10/23/2005 16:06'!
removeSharedPool: aSharedPool 
	sharedPools ifNil: [sharedPools _ subject sharedPools].
	sharedPools _ sharedPools copyWithout: aSharedPool! !

!ClassEditor methodsFor: 'initialize-release' stamp: 'cwp 10/23/2005 15:36'!
setSubject: aClass
	subject _ aClass.
	! !

!ClassEditor methodsFor: 'reflecting' stamp: 'cwp 10/24/2005 18:57'!
sharedPools
	^ sharedPools ifNil: [subject sharedPools]! !

!ClassEditor methodsFor: 'accessing' stamp: 'cwp 10/23/2005 01:54'!
subject
	^ subject! !

!ClassEditor methodsFor: 'reflecting' stamp: 'cwp 10/24/2005 07:54'!
superclass
	^ superclass ifNil: [self subject superclass]! !

!ClassEditor methodsFor: 'reflecting' stamp: 'cwp 10/24/2005 07:54'!
superclass: aClass
	superclass _ aClass! !

Object subclass: #ClassFormat
	instanceVariableNames: 'instSize instSpec cClassIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Editors'!

!ClassFormat commentStamp: 'cwp 9/25/2004 16:07' prior: 0!
I am an expert on the way the Squeak VM encodes class formats. My instance variable, 'bits', is identical in format to the 'format' instance variable of Behavior. I have methods for interpreting those bits. The class format integer is 18 bits in length. The instSize field is split into two parts for backwards compatibility.

<2 bits=instSize//64> <5 bits=cClass> <4 bits=instSpec> <6 bits=instSize\\64> <1 bit=0>

- instSize: the number of named instance variables
- cClass: the index into the compact class table
- instSpec: values indicating how instances are organized
	0 - no state
	1 - named instance variables only
	2 - variable number of object pointers
	3 - named instance variables, variable number of object pointers
	4 - variable number of weak object pointers
	6 - variable number of words
	8 - variable number of bytes!

!ClassFormat class methodsFor: 'instance creation' stamp: 'cwp 9/26/2004 18:14'!
bytes
	^ self size: 0 spec: self bytesSpec index: 0! !

!ClassFormat class methodsFor: 'storage specifications' stamp: 'cwp 9/26/2004 17:52'!
bytesSpec
	^ 8! !

!ClassFormat class methodsFor: 'instance creation' stamp: 'cwp 9/26/2004 17:41'!
fromBits: bits
	^ self
		size: (((bits bitShift: -10) bitAnd: 192) + ((bits bitShift: -1) bitAnd: 63) - 1)
		spec: ((bits bitShift: -7) bitAnd: 15)
		index: ((bits bitShift: -11) bitAnd: 31)! !

!ClassFormat class methodsFor: 'instance creation' stamp: 'cwp 9/26/2004 18:12'!
indexed
	^ self indexed: 0! !

!ClassFormat class methodsFor: 'storage specifications' stamp: 'cwp 9/26/2004 17:57'!
indexedSpec
	^ 2! !

!ClassFormat class methodsFor: 'storage specifications' stamp: 'cwp 9/26/2004 18:08'!
indexedSpec: size
	^ size = 0 ifTrue: [self indexedSpec] ifFalse: [self namedAndIndexedSpec]! !

!ClassFormat class methodsFor: 'instance creation' stamp: 'cwp 9/26/2004 18:03'!
indexed: size
	^ self 
		size: size
		spec: (self indexedSpec: size)
		index: 0! !

!ClassFormat class methodsFor: 'instance creation' stamp: 'cwp 9/26/2004 17:59'!
named
	^ self named: 0! !

!ClassFormat class methodsFor: 'storage specifications' stamp: 'cwp 9/26/2004 17:57'!
namedAndIndexedSpec
	^ 3! !

!ClassFormat class methodsFor: 'storage specifications' stamp: 'cwp 9/26/2004 17:55'!
namedSpec
	^ 1! !

!ClassFormat class methodsFor: 'instance creation' stamp: 'cwp 9/26/2004 18:06'!
named: size
	^ self
		size: size
		spec: self namedSpec
		index: 0! !

!ClassFormat class methodsFor: 'instance creation' stamp: 'cwp 9/26/2004 18:16'!
noState
	^ self size: 0 spec: self noStateSpec index: 0! !

!ClassFormat class methodsFor: 'storage specifications' stamp: 'cwp 9/26/2004 17:55'!
noStateSpec
	^ 0! !

!ClassFormat class methodsFor: 'instance creation' stamp: 'cwp 9/26/2004 17:41'!
size: instSize spec: instSpec index: cClassIndex
	^ self new setSize: instSize spec: instSpec index: cClassIndex! !

!ClassFormat class methodsFor: 'instance creation' stamp: 'cwp 9/26/2004 18:18'!
weak
	^ self size: 0 spec: self weakSpec index: 0! !

!ClassFormat class methodsFor: 'storage specifications' stamp: 'cwp 9/26/2004 17:52'!
weakSpec
	^ 4! !

!ClassFormat class methodsFor: 'instance creation' stamp: 'cwp 9/26/2004 18:17'!
words
	^ self size: 0 spec: self wordsSpec index: 0! !

!ClassFormat class methodsFor: 'storage specifications' stamp: 'cwp 9/26/2004 17:52'!
wordsSpec
	^ 6! !

!ClassFormat methodsFor: 'accessing' stamp: 'cwp 9/26/2004 17:37'!
bits
	"Based on ClassBuilder>>format:variable:words:pointers:weak - ar 7/11/1999 06:39 "

	| bits |
	bits _ (instSize+1) // 64. 	
	bits _ (bits bitShift: 5) + cClassIndex.
	bits _ (bits bitShift: 4) + instSpec.
	bits _ (bits bitShift: 6) + ((instSize+1)\\64).  "+1 since prim instSize field includes header"
	bits _ (bits bitShift: 1). "This shift plus integer bit lets wordSize work like byteSize"
	^ bits! !

!ClassFormat methodsFor: 'accessing' stamp: 'cwp 9/26/2004 17:36'!
cClassIndex
	^ cClassIndex! !

!ClassFormat methodsFor: 'converting' stamp: 'cwp 9/26/2004 18:15'!
indexedSubclass: nInstVars
	| newSize |
	self isPointers ifFalse: [self error: 'Can''t make a pointer subclass.'].
	newSize _ self instSize + nInstVars.
	newSize > 254 ifTrue: [self error: 'Too many instance variables'].
	
	^ self class indexed: newSize! !

!ClassFormat methodsFor: 'accessing' stamp: 'cwp 9/26/2004 17:37'!
instSize
	^ instSize! !

!ClassFormat methodsFor: 'accessing' stamp: 'cwp 9/26/2004 17:37'!
instSpec
	^ instSpec! !

!ClassFormat methodsFor: 'testing' stamp: 'cwp 9/25/2004 13:47'!
isBits
	"Answer whether the receiver contains just bits (not pointers)."

	^ self instSpec >= 6! !

!ClassFormat methodsFor: 'testing' stamp: 'cwp 9/25/2004 13:48'!
isBytes
	"Answer whether the receiver has 8-bit instance variables."

	^ self instSpec >= 8! !

!ClassFormat methodsFor: 'testing' stamp: 'cwp 9/25/2004 13:48'!
isFixed
	"Answer whether the receiver does not have a variable (indexable) part."

	^self isVariable not! !

!ClassFormat methodsFor: 'testing' stamp: 'cwp 9/25/2004 13:48'!
isPointers
	"Answer whether the receiver contains just pointers (not bits)."

	^self isBits not! !

!ClassFormat methodsFor: 'testing' stamp: 'cwp 9/25/2004 13:48'!
isVariable
	"Answer whether the receiver has indexable variables."

	^ self instSpec >= 2! !

!ClassFormat methodsFor: 'testing' stamp: 'cwp 9/25/2004 13:48'!
isWeak
	"Answer whether the receiver has contains weak references."
	^ self instSpec = 4! !

!ClassFormat methodsFor: 'testing' stamp: 'cwp 9/25/2004 13:49'!
isWords
	"Answer whether the receiver has 32-bit instance variables."

	^ self isBytes not! !

!ClassFormat methodsFor: 'converting' stamp: 'cwp 9/26/2004 18:09'!
namedSubclass: nInstVars 
	| newSize |
	newSize := self instSize + nInstVars.
	newSize > 254 ifTrue: [self error: 'Too many instance variables'].
	^self class named: newSize! !

!ClassFormat methodsFor: 'initializing' stamp: 'cwp 9/26/2004 17:39'!
setSize: size spec: spec index: index
	instSize _ size.
	instSpec _ spec.
	cClassIndex _ index.! !

!ClassFormat methodsFor: 'converting' stamp: 'cwp 9/26/2004 18:28'!
withInstSize: aSmallInteger
	self isPointers ifFalse: [self error: 'Only pointer formats can have named variables'].
	^ self class size: aSmallInteger spec: self instSpec index: self cClassIndex! !

Object subclass: #EditorSubject
	instanceVariableNames: 'alpha beta'
	classVariableNames: 'Aleph Beth'
	poolDictionaries: 'EditorSubjectPool'
	category: 'SystemEditor-Tests'!

EditorSubject class
	instanceVariableNames: 'omega'!

!EditorSubject class methodsFor: 'accessing class vars' stamp: 'cwp 10/24/2005 18:38'!
aleph
	^ Aleph ! !

!EditorSubject class methodsFor: 'accessing class vars' stamp: 'cwp 10/24/2005 18:38'!
aleph: anObject
	Aleph _ anObject! !

Object subclass: #MethodDictionaryEditor
	instanceVariableNames: 'subject additions removals'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Editors'!

!MethodDictionaryEditor class methodsFor: 'instance creation' stamp: 'cwp 10/25/2005 22:48'!
on: aMethodDictionary 
	^ self new setSubject: aMethodDictionary! !

!MethodDictionaryEditor methodsFor: 'editing' stamp: 'cwp 10/25/2005 22:51'!
add: aModifiedMethod 
	additions at: aModifiedMethod selector put: aModifiedMethod! !

!MethodDictionaryEditor methodsFor: 'testing' stamp: 'cwp 10/25/2005 22:54'!
includesSelector: aSymbol 
	^ (additions includesKey: aSymbol)
		or: [subject includesKey: aSymbol]! !

!MethodDictionaryEditor methodsFor: 'editing' stamp: 'cwp 10/25/2005 23:08'!
remove: aSymbol 
	additions removeKey: aSymbol ifAbsent: [].
	removals add: aSymbol! !

!MethodDictionaryEditor methodsFor: 'initialization' stamp: 'cwp 10/25/2005 23:04'!
setSubject: aMethodDictionary
	subject _ aMethodDictionary.
	additions _ IdentityDictionary new.
	removals _ IdentitySet new.! !

Object subclass: #ModifiedMethod
	instanceVariableNames: 'source category stamp requestor log'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Editors'!

!ModifiedMethod class methodsFor: 'as yet unclassified' stamp: 'cwp 10/25/2005 21:36'!
source: source classified: catString stamp: stamp notifying: requestor logging: aBoolean
	^ self new
		setSource: source
		classified: catString
		stamp: stamp
		notifying: requestor
		log: aBoolean! !

!ModifiedMethod methodsFor: 'as yet unclassified' stamp: 'cwp 10/25/2005 22:50'!
selector
	^ Parser new parseSelector: source! !

!ModifiedMethod methodsFor: 'as yet unclassified' stamp: 'cwp 10/25/2005 22:47'!
setSource: sourceString classified: catString stamp: timestamp notifying: anObject log: aBoolean
	source _ sourceString.
	category _ catString.
	stamp _ timestamp.
	requestor _ anObject.
	log _ aBoolean ! !
